@@ -42,10 +42,10 @@
 #include "document.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
-#include "libgcpcanvas/gcp-canvas-line.h"
+#include "libgcpcanvas/gcp-canvas-bpath.h"
 #include "libgcpcanvas/gcp-canvas-rich-text.h"
 #include <math.h>
-#include <chemistry/element.h>
+#include <gcu/element.h>
 
 using namespace gcu;
 
@@ -345,68 +345,86 @@ void gcpAtom::Add(GtkWidget* w)
 	pData->Items[this] = group;
 	m_width /= pData->ZoomFactor;
 	m_height /= pData->ZoomFactor;
-	char charge = GetCharge();
-	if (charge)
-		AddChargeItem (pData, group);
+	AddChargeItem (pData, group);
 }
 
 void gcpAtom::AddChargeItem(gcpWidgetData* pData, GnomeCanvasGroup* group)
 {
 	double x, y;
-	int align = GetChargePosition(x, y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
-	switch (align)
-	{
-		case -2:
-			y += 6.0;//FIXME: change 6.0 end other constants to variables
-			break;
-		case -1:
-			x -= 6.0;
-			break;
-		case 1:
-			x += 6.0;
-			break;
-		case 2:
-			y -= 6.0;
-			break;
-	}
-	GnomeCanvasGroup* chargegroup = GNOME_CANVAS_GROUP(gnome_canvas_item_new(group, gnome_canvas_group_ext_get_type(), NULL));
-	g_object_set_data ((GObject*)group, "charge", chargegroup);
-	gnome_canvas_item_new(
-					chargegroup,
-					gnome_canvas_ellipse_ext_get_type(),
-					"width_units", 1.0,
-					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
-					"x1", x - 4.0,
-					"x2", x + 4.0,
-					"y1", y - 4.0,
-					"y2", y + 4.0,
-					NULL);
-	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
-	points->coords[0] = x - 2.5;
-	points->coords[1] = points->coords[3] = y;
-	points->coords[2] = x + 2.5;
-	gnome_canvas_item_new(
-					chargegroup,
-					gnome_canvas_line_ext_get_type(),
-					"points", points,
-					"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-					"width_units", 1.0,
-					NULL);
-	if (GetCharge() > 0)
-	{	
-		points->coords[0] = points->coords[2] = x;
-		points->coords[1] = y - 2.5;
-		points->coords[3] = y + 2.5;
-		gnome_canvas_item_new(
-						chargegroup,
-						gnome_canvas_line_ext_get_type(),
-						"points", points,
-						"fill_color", (pData->IsSelected(this))? SelectColor: Color,
+	GnomeCanvasItem* item;
+	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
+	if (!GetCharge()) {
+		GetCoords (&x, &y);
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_ellipse_ext_get_type(),
+						"width_units", 0.,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"x1", x - 1.,
+						"x2", x + 1.,
+						"y1",  y - 1.,
+						"y2", y + 1.,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+		g_object_set_data ((GObject*)group, "circle", item);
+		gnome_canvas_path_def_moveto (path, x - 0.01, y);
+		gnome_canvas_path_def_lineto (path, x + 0.01, y);
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_bpath_ext_get_type(),
+						"bpath", path,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"width_units", 0.01,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+		g_object_set_data ((GObject*)group, "charge", item);
+	} else {
+		int align = GetChargePosition(x, y);
+		x *= pData->ZoomFactor;
+		y *= pData->ZoomFactor;
+		switch (align)
+		{
+			case -2:
+				y += 6.0;//FIXME: change 6.0 end other constants to variables
+				break;
+			case -1:
+				x -= 6.0;
+				break;
+			case 1:
+				x += 6.0;
+				break;
+			case 2:
+				y -= 6.0;
+				break;
+		}
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_ellipse_ext_get_type(),
+						"width_units", 1.0,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"x1", x - 4.0,
+						"x2", x + 4.0,
+						"y1", y - 4.0,
+						"y2", y + 4.0,
+						NULL);
+		g_object_set_data ((GObject*)group, "circle", item);
+		gnome_canvas_path_def_moveto (path, x - 2.5, y);
+		gnome_canvas_path_def_lineto (path, x + 2.5, y);
+		if (GetCharge() > 0)
+		{	
+			gnome_canvas_path_def_moveto (path, x, y - 2.5);
+			gnome_canvas_path_def_lineto (path, x, y + 2.5);
+		}
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_bpath_ext_get_type(),
+						"bpath", path,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 						"width_units", 1.0,
 						NULL);
+		g_object_set_data ((GObject*)group, "charge", item);
 	}
+	gnome_canvas_path_def_unref (path);
 }
 
 void gcpAtom::Update(GtkWidget* w)
@@ -417,13 +435,11 @@ void gcpAtom::Update(GtkWidget* w)
 	GetCoords(&x, &y);
 	x *= pData->ZoomFactor;
 	y *= pData->ZoomFactor;
-	GnomeCanvasGroup *chargegroup, *group = pData->Items[this];
-	void *obj = g_object_get_data(G_OBJECT(group), "symbol");
+	GnomeCanvasGroup *group = pData->Items[this];
 	if (m_Changed) {
-		if (group) gtk_object_destroy (GTK_OBJECT (group));
-		Add (w);
+		BuildItems (pData);
 	} else {
-		if (obj) {
+		if ((GetZ() != 6) || (GetBondsNumber() == 0)) {
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "symbol")),
 								"x", x - m_lbearing,
 								"y", y - m_ascent + m_CHeight,
@@ -435,6 +451,10 @@ void gcpAtom::Update(GtkWidget* w)
 								"y2", y  - m_ascent + m_CHeight + m_text_height + pData->Padding,
 								NULL);
 		} else {
+			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "symbol")),
+								"x", x,
+								"y", y,
+								NULL);
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 									"x1", x - 3,
 									"y1", y - 3,
@@ -442,11 +462,70 @@ void gcpAtom::Update(GtkWidget* w)
 									"y2", y + 3,
 								NULL);
 		}
-		chargegroup = (GnomeCanvasGroup*) g_object_get_data(G_OBJECT(group), "charge");
-		if (chargegroup) gtk_object_destroy(GTK_OBJECT(chargegroup));
-		char charge = GetCharge();
-		if (charge) AddChargeItem(pData, group);
 	}
+	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
+	void* item;
+	if (!GetCharge()) {
+		item = g_object_get_data(G_OBJECT(group), "circle");
+		g_object_set (item,
+						"width_units", 0.,
+						"x1", x - 1,
+						"x2", x + 1,
+						"y1", y - 1,
+						"y2", y + 1,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+		gnome_canvas_path_def_moveto (path, x - 0.01, y);
+		gnome_canvas_path_def_lineto (path, x + 0.01, y);
+		item = g_object_get_data(G_OBJECT(group), "charge");
+		g_object_set (item,
+						"bpath", path,
+						"width_units", 0.,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+	} else {
+		int align = GetChargePosition(x, y);
+		x *= pData->ZoomFactor;
+		y *= pData->ZoomFactor;
+		switch (align)
+		{
+			case -2:
+				y += 6.0;//FIXME: change 6.0 end other constants to variables
+				break;
+			case -1:
+				x -= 6.0;
+				break;
+			case 1:
+				x += 6.0;
+				break;
+			case 2:
+				y -= 6.0;
+				break;
+		}
+		item = g_object_get_data(G_OBJECT(group), "circle");
+		g_object_set (item,
+						"width_units", 1.0,
+						"x1", x - 4.0,
+						"x2", x + 4.0,
+						"y1", y - 4.0,
+						"y2", y + 4.0,
+						NULL);
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+		gnome_canvas_path_def_moveto (path, x - 2.5, y);
+		gnome_canvas_path_def_lineto (path, x + 2.5, y);
+		if (GetCharge() > 0)
+		{	
+			gnome_canvas_path_def_moveto (path, x, y - 2.5);
+			gnome_canvas_path_def_lineto (path, x, y + 2.5);
+		}
+		item = g_object_get_data(G_OBJECT(group), "charge");
+		g_object_set (item,
+						"bpath", path,
+						"width_units", 1.0,
+						NULL);
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+	}
+	gnome_canvas_path_def_unref (path);
 }
 
 int gcpAtom::GetChargePosition(double& x, double& y)
@@ -573,35 +652,21 @@ void gcpAtom::SetSelected(GtkWidget* w, int state)
 {
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
-	gchar* color;
+	gchar *color, *chargecolor;
 	
 	switch (state)
 	{	
-		case SelStateUnselected: color = "white"; break;
-		case SelStateSelected: color = SelectColor; break;
-		case SelStateUpdating: color = AddColor; break;
-		case SelStateErasing: color = DeleteColor; break;
-	}
-	GList* il = group->item_list;
-	while (il)
-	{
-		if (!GNOME_IS_CANVAS_RICH_TEXT_EXT(il->data))
-		{
-			if (GNOME_IS_CANVAS_GROUP_EXT(il->data))
-			{
-				GList* il1 = ((GnomeCanvasGroup*)il->data)->item_list;
-				while (il1)
-				{
-					if (GNOME_IS_CANVAS_LINE_EXT(il1->data))
-						g_object_set(G_OBJECT(il1->data), "fill_color", color, NULL);
-					else g_object_set(G_OBJECT(il1->data), "outline_color", color, NULL);
-					il1 = il1->next;
-				}
-			}
-			else g_object_set(G_OBJECT(il->data), "fill_color", color, NULL);
-		}
-		il = il->next;
+		case SelStateUnselected: color = "white"; chargecolor = "black"; break;
+		case SelStateSelected: chargecolor = color = SelectColor; break;
+		case SelStateUpdating: chargecolor = color = AddColor; break;
+		case SelStateErasing: chargecolor = color = DeleteColor; break;
 	}
+	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
+				"fill_color", color, NULL);
+	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "circle")),
+				"outline_color", chargecolor, NULL);
+	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "charge")),
+				"outline_color", chargecolor, NULL);
 }
 
 bool gcpAtom::AcceptNewBonds(int nb)
@@ -613,3 +678,109 @@ void gcpAtom::AddToMolecule(gcpMolecule* Mol)
 {
 	Mol->AddAtom(this);
 }
+
+void gcpAtom::BuildItems (gcpWidgetData* pData)
+{
+	GnomeCanvasGroup* group = pData->Items[this];
+	void* item;
+	GtkTextIter start, end;
+	double dFontHeight = pData->View->GetFontHeight();
+	PangoContext* pc = pData->View->GetPangoContext();
+	double x, y;
+	m_width =  m_height = 2.0 * pData->Padding;
+	GetCoords(&x, &y);
+	x *= pData->ZoomFactor;
+	y *= pData->ZoomFactor;
+	if ((GetZ() != 6) || (GetBondsNumber() == 0))
+	{
+		int sw, hw = 0, nw = 0;
+		int index;
+		char *hs;
+		gtk_text_buffer_get_bounds(m_buf, &start, &end);
+		gtk_text_buffer_delete(m_buf, &start, &end);
+		const gchar* symbol = GetSymbol();
+		gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, symbol, strlen(symbol),pData->View->GetFontName(), NULL);
+		PangoLayout *pl = pango_layout_new(pc);
+		pango_layout_set_text(pl, symbol, strlen(symbol));
+		PangoRectangle rect;
+		pango_layout_get_extents(pl, NULL, &rect);
+		m_width = sw = rect.width / PANGO_SCALE;
+		m_height = rect.height / PANGO_SCALE;
+		m_text_height = dFontHeight;
+		int n = GetAttachedHydrogens();
+		if (n > 0)
+		{
+			pango_layout_set_text(pl, "H", 1);
+			PangoRectangle rect;
+			pango_layout_get_extents(pl, NULL, &rect);
+			hw = rect.width / PANGO_SCALE;
+			index = (m_HPos)? strlen(symbol): 0;
+			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index);
+			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, "H", 1,pData->View->GetFontName(), NULL);
+		}
+		g_object_unref(pl);
+		if (n > 1)
+		{
+			pango_context_set_font_description(pc, pData->View->GetPangoSmallFontDesc());
+			pl = pango_layout_new(pc);
+			hs = g_strdup_printf("%d", n);
+			pango_layout_set_text(pl, hs, strlen(hs));
+			pango_layout_get_extents(pl, NULL, &rect);
+			nw = rect.width / PANGO_SCALE;
+			g_object_unref(pl);
+			pango_context_set_font_description(pc, pData->View->GetPangoFontDesc());
+			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
+			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, hs, strlen(hs),pData->View->GetSmallFontName(), "subscript", NULL);
+			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
+			gtk_text_buffer_get_iter_at_offset(m_buf, &end, index + 1 + strlen (hs));
+			gtk_text_buffer_remove_tag_by_name(m_buf, pData->View->GetFontName(), &start, &end);
+			m_text_height += 2;
+		}
+		m_length = sw + hw + nw;
+		m_lbearing = (m_HPos)? sw / 2: sw / 2 + hw + nw;
+
+		item = g_object_get_data (G_OBJECT (group), "rect");
+		g_object_set (item,
+							"x1", x - m_lbearing - pData->Padding,
+							"y1", y  - m_ascent + m_CHeight - pData->Padding,
+							"x2", x - m_lbearing + m_length + pData->Padding,
+							"y2", y  - m_ascent + m_CHeight + m_text_height + pData->Padding,
+							NULL);
+		
+		item = g_object_get_data (G_OBJECT (group), "symbol");
+		g_object_set (item,
+							"x", x - m_lbearing,
+							"y", y - m_ascent + m_CHeight,
+							"width", m_length,
+							"height", m_text_height,
+							NULL);
+	}
+	else
+	{
+		item = g_object_get_data (G_OBJECT (group), "rect");
+		g_object_set (item,
+							"x1", x - 3,
+							"y1", y - 3,
+							"x2", x + 3,
+							"y2", y + 3,
+							NULL);
+		gtk_text_buffer_get_bounds(m_buf, &start, &end);
+		gtk_text_buffer_delete(m_buf, &start, &end);
+		item = g_object_get_data (G_OBJECT (group), "symbol");
+		m_length = m_text_height = 0;
+		g_object_set (item,
+							"x", x,
+							"y", y,
+							"width", m_length,
+							"height", m_text_height,
+							NULL);
+	}
+	m_width /= pData->ZoomFactor;
+	m_height /= pData->ZoomFactor;
+	if (m_Changed > 0) m_Changed--;
+}
+	
+double gcpAtom::GetYAlign ()
+{
+	return m_y;
+}
