@@ -32,10 +32,10 @@
 #include <string>
 #include <libxml/tree.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libbonobo.h>
 #include <libbonoboui.h>
+#include <gcu/document.h>
 #include "atom.h"
 #include "fragment.h"
 #include "bond.h"
@@ -53,7 +53,7 @@ using namespace std;
 class gcpView;
 class gcpApplication;
 class gcpDocPropDlg;
-class gcpDocument: public Object
+class gcpDocument: public Document
 {
 	//Constructor and destructor
 public:
@@ -117,6 +117,7 @@ public:
 	void SetEditable(bool editable) {m_bWriteable = editable; m_bUndoRedo = true;}
 	bool GetEditable() {return m_bWriteable;}
 	gcpApplication* GetApplication() {return m_pApp;}
+	bool IsDirty () {return m_bDirty;}
 
 private:
 	void RemoveAtom(gcpAtom* pAtom);
