@@ -648,3 +648,12 @@ bool gcpText::OnEvent(GdkEvent *event)
 	}
 	else return false;
 }
+
+void gcpText::Transform2D(Matrix2D& m, double x, double y)
+{
+	m_x += m_length / 2 - x;
+	m_y += m_height / 2 - m_ascent +  - y;
+	m.Transform (m_x, m_y);
+	m_x -= m_length / 2 - x;
+	m_y -= m_height / 2 - m_ascent - y;
+}
