@@ -96,6 +96,21 @@ void gcpTool::OnRelease(double x, double y, unsigned int state)
 	g_signal_emit_by_name(m_pWidget, "update_bounds");
 }
 
+bool gcpTool::OnRightButtonClicked (gcpView* pView, Object* pObject, double x, double y)
+{
+	m_pObject = pObject;
+	m_pView = pView;
+	m_pWidget = m_pView->GetWidget();
+	m_pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(m_pWidget), "data");
+	m_pGroup = gnome_canvas_root(GNOME_CANVAS(m_pWidget));
+	m_pBackground = m_pView->GetBackground();
+	m_dZoomFactor = m_pView->GetZoomFactor();
+	m_x = x;
+	m_y = y;
+	OnRightButtonClicked ();
+	m_pObject = NULL;
+}
+
 bool gcpTool::Activate(bool bState)
 {
 	if (bState)
@@ -136,6 +151,11 @@ void gcpTool::OnRelease()
 {
 }
 
+bool gcpTool::OnRightButtonClicked()
+{
+	return false;
+}
+
 void gcpTool::OnChangeState()
 {
 	if (m_bPressed)
