@@ -27,7 +27,6 @@
 #ifndef GCHEMPAINT_VIEW_H
 #define GCHEMPAINT_VIEW_H
 
-#include <libgnome/libgnome.h>
 #include <map>
 #include <list>
 #include <libgnomeprint/gnome-print.h>
@@ -110,6 +109,8 @@ private:
 	double m_DefaultStereoBondWidth;
 	double m_DefaultHashWidth;
 	double m_DefaultHashDist;
+	double m_DefaultArrowPadding;
+	double m_DefaultSignPadding;
 	int m_width, m_height;
 	double m_lastx, m_lasty;
 	GnomeCanvasRichTextExt* m_ActiveRichText;
