@@ -29,8 +29,6 @@
 
 #include "lib/tool.h"
 #include "lib/arrow.h"
-#include "lib/reaction.h"
-#include "lib/mesomery.h"
 
 class gcpArrowTool: public gcpTool
 {
